@@ -2,13 +2,13 @@ package Test::BrewBuild;
 use strict;
 use warnings;
 
-use Data::Dumper;
+use File::Copy;
 use File::Temp;
 use Logging::Simple;
-use Test::BrewBuild::BrewCommands;
 use Plugin::Simple default => 'Test::BrewBuild::Plugin::DefaultExec';
+use Test::BrewBuild::BrewCommands;
 
-our $VERSION = '1.02';
+our $VERSION = '1.04';
 
 my $log;
 my $bcmd;
@@ -23,25 +23,30 @@ sub new {
 
     $bcmd = Test::BrewBuild::BrewCommands->new($log);
 
-    my $plugin = $args{plugin} ? $args{plugin} : $ENV{TBB_PLUGIN};
+    $self->_set_plugin();
 
-    $log->_6("plugin param set to: " . defined $plugin ? $plugin : 'default');
-
-    $plugin = $self->plugins($plugin, can => ['brewbuild_exec']);
-
-    my $exec_plugin_sub = $plugin .'::brewbuild_exec';
-    $self->{exec_plugin} = \&$exec_plugin_sub;
-
-    $log->_6("using plugin $plugin");
+    $self->tempdir;
+    $log->_7("using temp bblog dir: " . $self->tempdir);
 
     return $self;
 }
+sub brew_info {
+    my $self = shift;
+
+    my $log = $log->child('brew_info');
+
+    my $brew_info = $bcmd->info;
+
+    $log->_6("brew info set to:\n$brew_info") if $brew_info;
+
+    return $brew_info;
+}
 sub perls_available {
     my ($self, $brew_info) = @_;
 
     my $log = $log->child('perls_available');
 
-    my @perls_available = $bcmd->available($brew_info);
+    my @perls_available = $bcmd->available($self->{args}{legacy}, $brew_info);
 
     $log->_6("perls available: " . join ', ', @perls_available);
 
@@ -53,41 +58,7 @@ sub perls_installed {
     my $log = $log->child('perls_installed');
     $log->_6("checking perls installed");
 
-    return $bcmd->installed($brew_info);
-}
-sub instance_remove {
-    my ($self, @perls_installed) = @_;
-
-    my $log = $log->child('instance_remove');
-
-    $log->_5("perls installed: " . join ', ', @perls_installed);
-    $log->_5("removing previous installs...");
-
-    my $remove_cmd = $bcmd->remove;
-
-    $log->_6("using $remove_cmd remove command");
-
-    for (@perls_installed){
-        my $ver = $^V;
-        $ver =~ s/v//;
-
-        if ($_ =~ /$ver$/){
-            $log->_5("skipping version we're using, $_");
-            next;
-        }
-
-        $log->_6("exec'ing $remove_cmd");
-
-        if ($bcmd->is_win) {
-            `$remove_cmd $_ 2>nul`;
-        }
-        else {
-            `$remove_cmd $_ 2>/dev/null`;
-
-        }
-    }
-
-    $log->_5("removal of existing perl installs complete...");
+    return $bcmd->installed($self->{args}{legacy}, $brew_info);
 }
 sub instance_install {
     my ($self, $new, $perls_available, $perls_installed) = @_;
@@ -98,10 +69,11 @@ sub instance_install {
 
     my @new_installs;
 
-    if ($self->{args}{version}->[0]){
-        for my $version (@{ $self->{args}{version} }){
+    if ($self->{args}{install}->[0]){
+        for my $version (@{ $self->{args}{install} }){
+            $version = "perl-$version" if ! $self->is_win && $version !~ /perl/;
             if (grep { $version eq $_ } @{ $perls_installed }){
-                $log->_5("$version is already installed... skipping");
+                $log->_6("$version is already installed... skipping");
                 next;
             }
             push @new_installs, $version;
@@ -110,14 +82,14 @@ sub instance_install {
     else {
         if ($new){
 
-            $log->_6("looking to install $new perl instance(s)");
+            $log->_5("looking to install $new perl instance(s)");
 
             while ($new > 0){
 
                 my $candidate = $perls_available->[rand @{ $perls_available }];
 
                 if (grep { $_ eq $candidate } @{ $perls_installed }) {
-                    $log->_5( "$candidate already installed... skipping" );
+                    $log->_6( "$candidate already installed... skipping" );
                     next;
                 }
 
@@ -128,11 +100,11 @@ sub instance_install {
     }
 
     if (@new_installs){
-        $log->_5("preparing to install..." . join ', ', @new_installs);
+        $log->_4("preparing to install..." . join ', ', @new_installs);
 
         for my $ver (@new_installs){
             $log->_0("installing $ver...");
-            $log->_6("...using cmd: $install_cmd");
+            $log->_5("...using cmd: $install_cmd");
             `$install_cmd $ver`;
         }
     }
@@ -140,55 +112,39 @@ sub instance_install {
         $log->_5("using existing versions only");
     }
 }
-sub results {
-    my $self = shift;
+sub instance_remove {
+    my ($self, @perls_installed) = @_;
 
-    my $log = $log->child('results');
+    my $log = $log->child('instance_remove');
 
-    local $SIG{__WARN__} = sub {};
+    $log->_6("perls installed: " . join ', ', @perls_installed);
+    $log->_0("removing previous installs...");
 
-    $log->_6("warnings trapped locally");
+    my $remove_cmd = $bcmd->remove;
 
-    my $result = $self->exec;
+    $log->_4( "using '$remove_cmd' remove command" );
 
-    $log->_7($result);
+    for my $installed_perl (@perls_installed){
 
-    my @ver_results = $result =~ /[Pp]erl-\d\.\d+\.\d+.*?(?:Successfully tested .*?\n|FAIL\n)/gs;
+        my $using = $bcmd->using( $self->brew_info );
 
-    $log->_6("got " . scalar @ver_results . " results");
-
-    my @pass;
-    my $fail = 0;
-
-    for (@ver_results){
-        my $ver;
-
-        if (/^([Pp]erl-\d\.\d+\.\d+)/){
-            $ver = $1;
+        if ($using eq $installed_perl) {
+            $log->_5( "not removing version we're using: $using" );
+            next;
         }
-        my $res;
 
-        if (/Successfully tested /){
-            $log->_6("$ver PASSED...");
-            $res = 'PASS';
+        $log->_5( "exec'ing $remove_cmd $installed_perl" );
+
+        if ($bcmd->is_win) {
+            `$remove_cmd $installed_perl 2>nul`;
         }
         else {
-            $log->_6("$ver FAILED...");
-            print $_;
-            $fail = 1;
-            last;
+            `$remove_cmd $installed_perl 2>/dev/null`;
+
         }
-
-        $log->_6("$ver :: $res");
-        push @pass, "$ver :: $res\n";
     }
 
-    if (! $fail) {
-        print "\n";
-        print $_ for @pass;
-    }
-
-    $log->_6(__PACKAGE__ ." run finished");
+    $log->_4("removal of existing perl installs complete...");
 }
 sub run {
     my $self = shift;
@@ -196,7 +152,7 @@ sub run {
     my $new = defined $self->{args}{new} ? $self->{args}{new} : 0;
 
     my $log = $log->child('run');
-    $log->_6("commencing run()");
+    $log->_5("commencing run()");
 
     my $brew_info = $self->brew_info;
 
@@ -206,27 +162,99 @@ sub run {
 
     my @perls_installed = $self->perls_installed($brew_info);
 
-    $log->_5("installed perls: " . join ', ', @perls_installed);
+    $log->_4("installed perls: " . join ', ', @perls_installed);
 
     if ($self->{args}{remove}){
         $self->instance_remove(@perls_installed);
 
     }
 
-    if ($new) {
+    if ($new || $self->{args}{install}) {
         $self->instance_install($new, \@perls_available, \@perls_installed);
     }
 
-    @perls_installed = $self->perls_installed($brew_info);
+    # refetch installed in case we have installed a new instance
+
+    @perls_installed = $self->perls_installed($self->brew_info);
 
     if (! $perls_installed[0]){
         $log->_0("no perls installed... exiting");
         print "no perls installed... exiting" if $log->level;
     }
     else {
-        $self->results();
+        if (! $self->{args}{notest}){
+            $self->results();
+        }
     }
 }
+sub results {
+    my $self = shift;
+
+    my $log = $log->child('results');
+
+    local $SIG{__WARN__} = sub {};
+    $log->_6("warnings trapped locally");
+
+    my $failed = 0;
+
+    my $results = $self->exec;
+
+    $log->_7("\n*****\n$results\n*****");
+
+    my @ver_results = $results =~ /
+        [Pp]erl-\d\.\d+\.\d+(?:_\w+)?\s+===
+        .*?
+        (?=(?:[Pp]erl-\d\.\d+\.\d+(?:_\w+)?\s+===|$))
+        /gsx;
+
+    $log->_5("got " . scalar @ver_results . " results");
+
+    my (@pass, @fail);
+
+    for my $result (@ver_results){
+        my $ver;
+
+        if ($result =~ /^([Pp]erl-\d\.\d+\.\d+)/){
+            $ver = $1;
+            $ver =~ s/[Pp]erl-//;
+        }
+        my $res;
+
+        if ($result =~ /Successfully tested / && $result !~ /FAIL/){
+            $log->_6("$ver PASSED...");
+            $res = 'PASS';
+            push @pass, "$ver :: $res\n";
+        }
+        else {
+            $log->_6("$ver FAILED...");
+            $res = 'FAIL';
+            $failed = 1;
+            push @fail, "$ver :: $res\n";
+
+            my $tested_mod = $self->{args}{plugin_arg};
+
+            if (defined $tested_mod){
+                $tested_mod =~ s/::/-/g;
+                open my $wfh, '>', "$self->{tempdir}/$tested_mod-$ver.bblog"
+                    or die $!;
+                print $wfh $result;
+                close $wfh;
+            }
+            else {
+                open my $wfh, '>', "$self->{tempdir}/$ver.bblog"  or die $!;
+                print $wfh $result;
+                close $wfh;
+            }
+        }
+    }
+
+    print "\n";
+    print "$self->{args}{plugin_arg}\n" if $self->{args}{plugin_arg};
+    print $_ for @pass;
+    print $_ for @fail;
+
+    $log->_5(__PACKAGE__ ." run finished");
+}
 sub exec {
     my $self = shift;
 
@@ -234,78 +262,98 @@ sub exec {
 
     $log->_6("creating temp file");
 
-    my $wfh = File::Temp->new(UNLINK => 1);
-    my $fname = $wfh->filename;
-
-    $log->_6("temp filename: $fname");
-    $log->_6("fetching instructions from the plugin");
-    $log->_6("instructions to be executed:");
-
-    my @exec_cmd = $self->{exec_plugin}->(@{ $self->{args}{args} });
-
-    for (@exec_cmd){
-        $log->_6($_);
-        print $wfh $_;
+    if ($self->{args}{plugin_arg}) {
+        $log->_5( "" .
+            "fetching instructions from the plugin with arg " .
+            $self->{args}{plugin_arg}
+        );
     }
-    close $wfh;
+    
+    my @exec_cmd = $self->{exec_plugin}->(
+        __PACKAGE__,
+        $self->log,
+        $self->{args}{plugin_arg}
+    );
 
-    $log->_6("temp file handle closed");
+    $log->_6("instructions to be executed:\n" . join ', ', @exec_cmd);
 
     my $brew = $bcmd->brew;
 
     if ($self->{args}{on}){
         my $vers = join ',', @{ $self->{args}{on} };
-        $log->_6("versions to run on: $vers");
-        $log->_6("exec'ing: $brew exec --with $vers perl $fname");
-
-        if ($bcmd->is_win){
-            return `$brew exec --with $vers perl $fname 2>nul`;
+        $log->_5("versions to run on: $vers");
+        $log->_5("exec'ing: $brew exec --with $vers " . join ', ', @exec_cmd);
 
+        my $wfh = File::Temp->new(UNLINK => 1);
+        my $fname = $wfh->filename;
+        open $wfh, '>', $fname or die $!;
+        for (@exec_cmd){
+            s/\n//g;
         }
-        else {
-            return `$brew exec --with $vers perl $fname 2>/dev/null`;
-        }
+        my $cmd = join ' && ', @exec_cmd;
+        $cmd = "system(\"$cmd\")";
+        print $wfh $cmd;
+        close $wfh;
+        return `$brew exec --with $vers perl $fname 2>$self->{tempdir}/stderr.bblog`;
     }
     else {
-        $log->_6("exec'ing: $brew exec perl $fname");
+        $log->_5("exec'ing: $brew exec:\n". join ', ', @exec_cmd);
 
-        if ($bcmd->is_win) {
-            return `$brew exec perl $fname 2>nul`;
+        if ($bcmd->is_win){
+
+            # all of this shit because berrybrew doesn't get the path right
+            # when calling ``berrybrew exec perl ...''
+
+            my %res_hash;
+
+            for (@exec_cmd){
+                my $res = `$brew exec $_ 2>$self->{tempdir}/stderr.bblog`;
+
+                my @results = $res =~ /
+                    [Pp]erl-\d\.\d+\.\d+(?:_\w+)?\s+===
+                    .*?
+                    (?=(?:[Pp]erl-\d\.\d+\.\d+(?:_\w+)?\s+===|$))
+                    /gsx;
+
+                for (@results){
+                    if ($_ =~ /
+                        ([Pp]erl-\d\.\d+\.\d+(?:_\w+)?\s+=+?)
+                        (\s+.*?)
+                        (?=(?:[Pp]erl-\d\.\d+\.\d+(?:_\w+)?\s+===|$))
+                        /gsx)
+                    {
+                        push @{ $res_hash{$1} }, $2;
+                    }
+                }
+            }
+
+            my $result;
+
+            for (keys %res_hash){
+                $result .= $_ . join '', @{ $res_hash{$_} };
+            }
+
+            return $result;
         }
         else {
-            return `$brew exec perl $fname 2>/dev/null`;
+            my $wfh = File::Temp->new(UNLINK => 1);
+            my $fname = $wfh->filename;
+            open $wfh, '>', $fname or die $!;
+            for (@exec_cmd){
+                s/\n//g;
+            }
+            my $cmd = join ' && ', @exec_cmd;
+            $cmd = "system(\"$cmd\")";
+            print $wfh $cmd;
+            close $wfh;
+            return `$brew exec perl $fname 2>$self->{tempdir}/stderr.bblog`;
         }
     }
 }
-sub brew_info {
+sub tempdir {
     my $self = shift;
-
-    my $log = $log->child('brew_info');
-
-    my $brew_info = $bcmd->available;
-
-    $log->_6("brew info set to:\n$brew_info") if $brew_info;
-
-    return $brew_info;
-}
-sub _create_log {
-    my ($self, $level) = @_;
-
-    $self->{log} = Logging::Simple->new(
-        name  => 'Test::BrewBuild',
-        level => $level,
-    );
-
-    $self->{log}->_6("in _create_log()");
-
-    if (defined $level && $level < 6){
-        $self->{log}->display(0);
-        $self->{log}->custom_display("-");
-        $self->{log}->_6("setting log level to $level");
-        $self->{log}->_6("log object created with level $level");
-    }
-
-    return $self->{log};
+    $self->{tempdir} = $self->{args}{tempdir} if defined $self->{args}{tempdir};
+    return $self->{tempdir} || '';
 }
 sub log {
     my $self = shift;
@@ -316,6 +364,38 @@ sub is_win {
     my $is_win = ($^O =~ /Win/) ? 1 : 0;
     return $is_win;
 }
+sub _create_log {
+    my ($self, $level) = @_;
+
+    $self->{log} = Logging::Simple->new(
+        name  => 'Test::BrewBuild',
+        level => defined $level ? $level : 0,
+    );
+
+    $self->{log}->_6("in _create_log()");
+
+    if ($self->{log}->level < 6){
+        $self->{log}->display(0);
+        $self->{log}->custom_display("-");
+        $self->{log}->_5("set log level to " . defined $level ? $level : 0);
+    }
+
+    return $self->{log};
+}
+sub _set_plugin {
+    my $self = shift;
+    my $log = $log->child('_set_plugin');
+    my $plugin = $self->{args}{plugin} ? $self->{args}{plugin} : $ENV{TBB_PLUGIN};
+
+    $log->_5("plugin param set to: " . defined $plugin ? $plugin : 'default');
+
+    $plugin = $self->plugins($plugin, can => ['brewbuild_exec']);
+
+    my $exec_plugin_sub = $plugin .'::brewbuild_exec';
+    $self->{exec_plugin} = \&$exec_plugin_sub;
+
+    $log->_4("successfully loaded $plugin plugin");
+}
 1;
 
 =head1 NAME
@@ -358,7 +438,7 @@ All unit tests are run against all installed instances.
     my %args = (
         debug   => undef,
         remove  => undef,
-        version => undef,
+        install => undef,
         new     => undef,
         plugin  => undef,
         on      => undef,
@@ -414,7 +494,7 @@ setup.
 
 =head2 instance_install
 
-If 'version' param is set, will install that specific version. If 'new' param
+If 'install' param is set, will install that specific version. If 'new' param
 is set to a positive integer, will install that many random versions of perl.
 
 =head2 instance_remove
@@ -450,6 +530,10 @@ we're working on.
 Developer method, returns an instance of the packages log object for creating
 child log objects.
 
+=head2 tempdir
+
+Sets up the object with a temporary directory that will be removed after run.
+
 =head1 AUTHOR
 
 Steve Bertrand, C<< <steveb at cpan.org> >>
