@@ -2,7 +2,9 @@ package Test::BrewBuild::BrewCommands;
 use strict;
 use warnings;
 
-our $VERSION = '1.01';
+use version;
+
+our $VERSION = '1.04';
 
 my $log;
 
@@ -23,41 +25,60 @@ sub brew {
     $log->child('brew')->_6("*brew cmd is: $brew");
     return $brew;
 }
+sub info {
+    my $self = shift;
+    return $self->is_win
+        ? `berrybrew available 2>nul`
+        : `perlbrew available 2>/dev/null`;
+}
 sub installed {
-    my ($self, $info) = @_;
+    my ($self, $legacy, $info) = @_;
 
     $log->child('installed')->_6("cleaning up perls installed");
 
     return if ! $info;
 
-    return $self->is_win
+    my @installed = $self->is_win
         ? $info =~ /(\d\.\d{2}\.\d(?:_\d{2}))(?!=_)\s+\[installed\]/ig
         : $info =~ /i.*?(perl-\d\.\d+\.\d+)/g;
 
+    if (! $legacy){
+        @installed = grep { /^(?:perl-)?\d\.(\d+)/; $1 >= 8 } @installed;
+    }
+
+    return @installed;
+
 }
-sub available {
+sub using {
     my ($self, $info) = @_;
 
+    $log->child( 'using' )->_6( "checking for which ver we're using" );
 
-    if ($info){
-
-        $log->child('available')->_6("determining available perls");
-
-        my @avail = $self->is_win
-            ? $info =~ /(\d\.\d+\.\d+_\d+)/g
-            : $info =~ /(perl-\d\.\d+\.\d+)/g;
-
-        return @avail;
+    if ($self->is_win) {
+        my @installed = $info =~ /(\d\.\d{2}\.\d(?:_\d{2}))(?!=_)\s+\[installed\]\*/ig;
+        return $installed[0];
     }
     else {
-
-        $log->child('available')->_6("generating available");
-
-        return $self->is_win
-            ? `berrybrew available 2>nul`
-            : `perlbrew available 2>/dev/null`;
+        my $using = version->parse($])->normal;
+        $using =~ s/v//;
+        $using = "perl-$using";
+        return $using;
     }
 }
+sub available {
+    my ($self, $legacy, $info) = @_;
+
+    $log->child('available')->_6("determining available perls");
+
+    my @avail = $self->is_win
+        ? $info =~ /(\d\.\d+\.\d+_\d+)/g
+        : $info =~ /(perl-\d\.\d+\.\d+)/g;
+
+    if (! $legacy){
+        @avail = grep { /^(?:perl-)?\d\.(\d+)/; $1 > 8 } @avail;
+    }
+    return @avail;
+}
 sub install {
     my $self = shift;
 
@@ -102,15 +123,23 @@ Returns a new Test::BrewBuild::BrewCommands object.
 
 Returns 'perlbrew' if on Unix, and 'berrybrew' if on Windows.
 
+=head2 info
+
+Returns the string result of *brew available.
+
 =head2 installed($info)
 
 Takes the output of '*brew available' in a string form. Returns the currently
 installed versions, formatted in a platform specific manner.
 
-=head2 available($info)
+=head2 available($legacy, $info)
 
 Similar to C<installed()>, but returns all perls available.
 
+=head2 using($info)
+
+Returns the current version of perl we're using.
+
 =head2 install
 
 Returns the current OS's specific *brew install command.
@@ -150,4 +179,4 @@ See L<http://dev.perl.org/licenses/> for more information.
 
 
 =cut
-
+ 
